@@ -6,6 +6,8 @@ Advanced Interface Switcher
 System menu plugin class
 '''
 
+#from __future__ import with_statement
+#import threading
 import gobject
 import gtk
 import re
@@ -23,6 +25,7 @@ from advifswlib.interfaces.GSM import GSMInterface
 from advifswlib.interfaces.BT import BTInterface
 from advifswlib.interfaces.WLAN import WLANInterface
 import advifswlib.util.cacher as cacher
+import advifswlib.util.worker as worker
 import advifswlib.util.config as config
 import advifswlib.util.locale as locale
 
@@ -37,6 +40,7 @@ class AdvancedInterfaceSwitcherMaemo5(StatusMenuItem):
   def __init__(self):
 
     StatusMenuItem.__init__(self)
+    #self.__ui_lock = threading.Lock()
     self.__lcl = locale.get()
     self.__interfaces = {}
     self.__iface_config_re = re.compile(r'settings/(\w+)/.+')
@@ -53,58 +57,86 @@ class AdvancedInterfaceSwitcherMaemo5(StatusMenuItem):
     self.__m_loop = DBusGMainLoop()
     self.__bus = dbus.SystemBus(mainloop = self.__m_loop, private = True)
     self.__available_ifaces = (GSMInterface, BTInterface, WLANInterface)#, FlightMode)
-    self.__smi_box = gtk.Table(len(self.__available_ifaces), 1, True)
-    self.add(self.__smi_box)
+    self.__smi_box = gtk.HBox(True)#gtk.Table(len(self.__available_ifaces), 1, True)
+    smi_vbox = gtk.VBox(False)
+    smi_vbox.pack_start(self.__smi_box)
+    #self.add(self.__smi_box)
+    self.add(smi_vbox)
 
     loaded_list = ''
-    ind = 0
-
+    active = 0
     for iface in self.__available_ifaces:
       key = iface.name.lower()
       if self.__config.get_value('settings/%s/enabled' % (key)):
         try:
-          obj = iface(self.__bus, self.process_sai,
-                      self.__show_settings_dialog)
+          obj = self.__init_iface(iface)
           self.__interfaces[key] = obj
-          self.__smi_box.attach(obj.get_button(),
-                                ind, ind + 1, 0, 1,
-                                gtk.EXPAND | gtk.FILL,
-                                gtk.SHRINK)
           loaded_list = '%s\n%s (%s)' % (loaded_list, obj.get_name(),
                                          obj.__class__)
-          ind += 1
+          active += 1
         except Exception, e:
           loaded_list = '%s\n%s\nError: %s\n' % (loaded_list, iface, e)
           pass
-    if not ind:
-      self.__settings_button = hildon.Button(gtk.HILDON_SIZE_AUTO_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT,
-                                             hildon.BUTTON_ARRANGEMENT_HORIZONTAL,
-                                             self.__gtk_swr.get_text(gtk.STOCK_PREFERENCES))
-      self.__settings_button.set_image(gtk.Image())
-      self.__settings_button.get_image().set_from_pixbuf(cacher.get().get('general_settings'))
-      self.__settings_button.set_alignment(0.5,0.5,1,1)
-      self.__settings_button.set_title_alignment(0.5,0.5)
-      self.__settings_button.connect('clicked', self.__settings_button_clicked)
-      self.__smi_box.attach(self.__settings_button,
-                            ind, len(self.__available_ifaces) - 1, 0, 1,
-                            gtk.EXPAND | gtk.FILL,
-                            gtk.SHRINK)
+      else:
+        self.__interfaces[key] = None
+    self.__setup_buttons()
+
+    self.__settings_button = hildon.Button(gtk.HILDON_SIZE_AUTO_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT,
+      hildon.BUTTON_ARRANGEMENT_VERTICAL,
+      self.__gtk_swr.get_text(gtk.STOCK_PREFERENCES),
+      self.app_name)
+    self.__settings_button.set_image(gtk.Image())
+    self.__settings_button.get_image().set_from_pixbuf(cacher.get().get('general_settings'))
+    self.__settings_button.set_alignment(0.5,0.5,1,1)
+    self.__settings_button.set_title_alignment(0.0,0.5)
+    self.__settings_button.connect('clicked', self.__settings_button_clicked)
+
+    smi_vbox.pack_start(self.__settings_button)
+      #self.__smi_box.attach(self.__settings_button,
+      #  ind, len(self.__available_ifaces) - 1, 0, 1,
+      #  gtk.EXPAND | gtk.FILL,
+      #  gtk.SHRINK)
     self.__settings_dialog = None
     self.__config_controls = {}
     print '\n%s\n%s' % (self.app_name, self.version)
-    print '\nLoaded (%s/%s)%s\n' % (ind, len(self.__available_ifaces), loaded_list)
+    print '\nLoaded (%s/%s)%s\n' % (active, len(self.__available_ifaces), loaded_list)
     self.show_all()
-#    self.__show_settings_dialog()
+    if active:
+      self.__settings_button.hide()
+    #self.__show_settings_dialog()
 
   def destroy(self):
     for iface in self.__interfaces:
-      self.__interfaces[iface].destroy()
+      if self.__interfaces[iface]:
+        self.__interfaces[iface].destroy()
     self.__config_controls.clear()
     if self.__settings_dialog:
       self.__settings_dialog.destroy()
     StatusMenuItem.destroy(self)
+    worker.get().shutdown()
     print '\n%s: destroyed' % (self.__class__.__name__)
 
+  def __init_iface(self, iface):
+    return iface(self.__bus, self.process_sai, self.__show_settings_dialog)
+
+  def __setup_buttons(self):
+    #with self.__ui_lock:
+#    self.__smi_box.attach(self.__interfaces[iface].get_button(),
+#                                ind, ind + 1, 0, 1,
+#                                gtk.EXPAND | gtk.FILL,
+#                                gtk.SHRINK)
+    #button = self.__interfaces[iface].get_button()
+    #self.__smi_box.pack_start(button)
+    #position = min(len(self.__smi_box.get_children()) - 1, self.__interfaces[iface].position)
+    #self.__smi_box.reorder_child(button, position)
+    children = self.__smi_box.get_children()
+    for child in children:
+      self.__smi_box.remove(child)
+    for iface in self.__available_ifaces:
+      iface_obj = self.__interfaces[iface.name.lower()]
+      if iface_obj:
+        self.__smi_box.pack_start(iface_obj.get_button())
+
   def __settings_button_clicked(self, button):
     button.get_toplevel().hide()
     gobject.idle_add(self.__show_settings_dialog, button)
@@ -189,7 +221,8 @@ class AdvancedInterfaceSwitcherMaemo5(StatusMenuItem):
           self.__config.set_value(config_key, value, True, self.__config_callback)
       #self.__config.save()
       for iface in self.__interfaces:
-        self.__interfaces[iface].update()
+        if self.__interfaces[iface]:
+          self.__interfaces[iface].update()
     self.__settings_dialog.hide()
     return False
 
@@ -200,9 +233,28 @@ class AdvancedInterfaceSwitcherMaemo5(StatusMenuItem):
         if iface_match:
           iface = iface_match.group(1)
           if iface in self.__interfaces:
-            self.__interfaces[iface].config_changed(
-              key.replace('settings/%s/' % iface, '', 1),
-              old_value, new_value)
+            subkey = key.replace('settings/%s/' % iface, '', 1)
+            if subkey == 'enabled':
+              if new_value:
+                iface_class = None
+                for a_iface_class in self.__available_ifaces:
+                  if a_iface_class.name.lower() == iface:
+                    iface_class = a_iface_class
+                    break
+                if iface_class:
+                  if not self.__interfaces[iface]:
+                    self.__interfaces[iface] = self.__init_iface(iface_class)
+              else:
+                self.__smi_box.remove(self.__interfaces[iface].get_button())
+                if not len(self.__smi_box.get_children()):
+                  self.__settings_button.show()
+            if self.__interfaces[iface]:
+              self.__interfaces[iface].config_changed(
+                subkey, old_value, new_value)
+              if subkey == 'enabled'and new_value:
+                self.__settings_button.hide()
+                self.__setup_buttons()
+                self.__interfaces[iface].get_button().show_all()
 
   def process_sai(self, iname, image='', add=True):
     if add:
