@@ -18,6 +18,8 @@ except Exception, e:
 # It allows to run the widget as a standalone process.
 if __name__ == "__main__":
   import sys
+#  import advifswlib.util.worker as worker
+
   plugin_id = "advanced-interface-switcher.console"
   if len(sys.argv) == 2:
     try:
@@ -42,3 +44,5 @@ if __name__ == "__main__":
       gtk.main()
     except (KeyboardInterrupt, SystemExit):
       obj.destroy()
+#      worker.get().shutdown()
+      raise
